/***************************************************************************
 *   Copyright (C) 2004 by Michael Rex                                     *
 *   me@rexi.org                                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef SETIPARSER_H
#define SETIPARSER_H

#include <stdbool.h>

typedef enum
{
    DOWNLOADING = 1,
    READY = 2,
    UPLOADING = 4,
    UPLOADED = 5,
    
    RUNNING = 999
}  workunit_state;

typedef struct
{
    char *name;
    char *result_name;
    workunit_state state;
    double deadline;
    char *deadline_str;
    bool readytoreport;
    double progress;
    double pph;
    int slot;
    double cputime;
    char *cputime_str;
    double final_cpu;
    char *final_cpu_str;
    char *timeleft;
    char *timetotal;
    char *finishedon;
} workunit_t;

typedef struct
{
/* host info */
    char *hostname;
    char *ip;
    int number_cpus;
    double float_mips;
    double int_mips;
    char *cpu_model;
    char *cpu_vendor;
    char *os_name;
    char *os_version;
    int timezone;
    double memory;
    double disk_total;
    double disk_free;
    double calculated;
    char *calculated_str;

/* project info */
    char *project_name;
    char *project_url;
    char *user_name;
    double user_total_credit;
    double user_avg_credit;
    double host_total_credit;
    double host_avg_credit;
    double exp_cputime;
    char *exp_cputime_str;

/* workunit statistics */
    int workunit_count;
    int finished_count;
    int active_count;
    
/* network stats */
    double upload;
    double download;
    
/* miscellaneous */
    char *platform;
    workunit_t workunits[128];
    
} client_state;

void parse_client_state(const char *path, client_state *p);
char *get_state_name(int state);
char *short_time_string(double sec);
client_state *get_empty_struct(client_state *p);

#endif
