/***************************************************************************
 *   Copyright (C) 2004 by Michael Rex                                     *
 *   me@rexi.org                                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_GETOPT_LONG
#include <getopt.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>
#include <assert.h>
#include <unistd.h>

#include "setiparser.h"
#include "ncursesui.h"
#include "configuration.h"
#include "configparser.h"
#include "clientmanager.h"
#include "version.h"


#define RCFILE ".setistatrc"

#define BOINC_LINUX_X86     "boinc_4.13_i686-pc-linux-gnu"
#define BOINC_SOLARIS_SPARC "boinc_4.13_sparc-sun-solaris2.7"
#define BOINC_MACOSX_PPC    "boinc_4.13_powerpc-apple-darwin"

#if defined(SYSTEM_LINUX)
    #define BOINC_CLIENT BOINC_LINUX_X86
#elif defined(SYSTEM_SUNOS)
    #define BOINC_CLIENT BOINC_SOLARIS_SPARC
#elif defined(SYSTEM_DARWIN)
    #define BOINC_CLIENT BOINC_MACOSX_PPC
#else
    #define BOINC_CLIENT BOINC_LINUX_X86
#endif

/* Prototypes */
static void parse_commandline(int argc, char *argv[], config_items *cfg);
static void show_help_message(void);
static void show_version_message(void);



int main(int argc, char *argv[])
{
    config_items cfg = {
        getenv("HOME"),
        BOINC_CLIENT,
        NULL,
        20,
        false,
        "/tmp/boinc.log",
        false
    };

    parse_rc_file(RCFILE, &cfg);
    parse_commandline(argc, argv, &cfg);
    start_ncurses(&cfg);

    return EXIT_SUCCESS;
}


/**
 *
 */
static void parse_commandline(int argc, char *argv[], config_items *cfg)
{
    int current_arg = 0; /* index of current argument */

#ifdef HAVE_GETOPT_LONG
    /* struct with valid command line arguments */
    struct option longoptions[]=
    {
        {"help",        no_argument,       NULL, 'h'},
        {"version",     no_argument,       NULL, 'V'},
        {"path",        required_argument, NULL, 'p'},
        {"start",       no_argument,       NULL, 's'},
        {"stop",        no_argument,       NULL, 'k'},
        {"refresh",     required_argument, NULL, 'r'},
        {"monochrome",  no_argument,       NULL, 'm'},
        {NULL, 0, NULL, 0}
    };
#endif
    
    /* process command line arguments */
#ifdef HAVE_GETOPT_LONG
    while((current_arg = getopt_long(argc, argv, "hVp:mskr:", longoptions, NULL)) != -1)
#else
    while((current_arg = getopt(argc, argv, "hVp:mskr:")) != -1)
#endif
    {
        switch(current_arg)
        {
            case 's': /* start client program */
                if(start_client(cfg->boincdir, cfg->clientcmd, cfg->clientargs, cfg->logfile, cfg->appendlog) == -1)
                {
                
                }
                exit(EXIT_SUCCESS);
                break;
            case 'k': /* stop client program */
                if(stop_client(cfg->boincdir) == -1)
                {
                
                }
                exit(EXIT_SUCCESS);
                break;
            case 'p': /* set path */
                free(cfg->boincdir);
                cfg->boincdir = (char *)malloc(strlen(optarg) + 1);
                assert(cfg->boincdir != NULL);
                strcpy(cfg->boincdir, optarg);
            case 'r': /* set update interval */
                cfg->refresh_rate = atoi(optarg);
                break;
            case 'h': /* show help */
                show_help_message();
                break;
            case 'V': /* show version information */
                show_version_message();
                break;
            case 'm': /* monochrome output */
                cfg->monochrome = true;
                break;
            case '?': /* show error message if argument wasn't valid */
                fprintf(stderr, "See \"setistat --help\" for more information.\n");
                exit(EXIT_FAILURE);
        }
    }
}

static void show_help_message(void)
{
    printf("Usage: setistat [OPTIONS]\n"
        "Monitors a SETI@home client and shows its progress.\n\n"
        "Options:\n"
        "   -p   --path         Specifies the directory the work unit is in\n"
        "   -s   --start        Start a SETI@home client to process the work unit\n"
        "   -k   --stop         Stop a running SETI@home client\n"
        "   -r   --refresh      Seconds before workunit data is reread and display is\n"
        "                       updated\n"
        "   -m   --monochrome   Do not use colors\n"
        "   -h   --help         Print help and exit\n"
        "   -V   --version      Print version and exit\n");

    exit(EXIT_SUCCESS);
}

static void show_version_message(void)
{
    printf("%s %s (%s)\n%s\n%s\n", PKG_NAME, PKG_VERSION, RELEASEDATE, COPYRIGHT, LICENSE);
    exit(EXIT_SUCCESS);
}
